﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            UltraGridBand band = this.ultraGrid1.DisplayLayout.Bands[0];
            this.ultraGrid1.CreationFilter = new ColHeaderSpannerCreationFilter(
                band.Columns["Column 1"], 
                band.Columns["Column 2"]);

            this.ultraGrid1.DisplayLayout.Override.AllowColSwapping = AllowColSwapping.WithinBand;

        }       

        private void ultraGrid1_BeforeColRegionScroll_1(object sender, BeforeColRegionScrollEventArgs e)
        {
            UltraGrid grid = (UltraGrid)sender;
            UIElement bandHeadersUIElement = grid.DisplayLayout.UIElement.GetDescendant(typeof(BandHeadersUIElement));
            if (bandHeadersUIElement != null)
                bandHeadersUIElement.DirtyChildElements(true);
        }
    }

    public class ColHeaderSpannerCreationFilter : IUIElementCreationFilter
    {
        private UltraGridColumn Column1 { get; set; }
        private UltraGridColumn Column2 { get; set; }

        public ColHeaderSpannerCreationFilter(UltraGridColumn column1, UltraGridColumn column2)
        {
            this.Column1 = column1;
            this.Column2 = column2;
        }

        public void AfterCreateChildElements(UIElement parent)
        {
            if (parent is BandHeadersUIElement)
            {
                HeaderUIElement headerElement1 = parent.GetDescendant(typeof(HeaderUIElement), this.Column1) as HeaderUIElement;
                HeaderUIElement headerElement2 = parent.GetDescendant(typeof(HeaderUIElement), this.Column2) as HeaderUIElement;

                // If boith of the elements are scrolled out of view, then do nothing. 
                if (headerElement1 == null &&
                    headerElement2 == null)
                {
                    return;
                }

                UIElement elementToExtend = headerElement1 != null ? headerElement1 : headerElement2;
                UIElement elementToRemove = headerElement1 != null ? headerElement2 : headerElement1;

                // Make one header element fill the space of both column headers. 
                if (elementToRemove != null)
                {
                    elementToExtend.Rect = Rectangle.Union(elementToExtend.Rect, elementToRemove.Rect);

                    // Remove the other element
                    parent.ChildElements.Remove(elementToRemove);
                }
            }
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            //Return false to let the grid know we did not handle the event. 
            return false;
        }
    }
}
